# To use this code, make sure you
#
#     import json
#
# and then, to convert JSON from a string, do
#
#     result = im_list_response_from_dict(json.loads(json_string))

from dataclasses import dataclass
from typing import Optional, Any, List, TypeVar, Callable, Type, cast


T = TypeVar("T")


def from_str(x: Any) -> str:
    assert isinstance(x, str)
    return x


def from_none(x: Any) -> Any:
    assert x is None
    return x


def from_union(fs, x):
    for f in fs:
        try:
            return f(x)
        except:
            pass
    assert False


def from_int(x: Any) -> int:
    assert isinstance(x, int) and not isinstance(x, bool)
    return x


def from_bool(x: Any) -> bool:
    assert isinstance(x, bool)
    return x


def from_list(f: Callable[[Any], T], x: Any) -> List[T]:
    assert isinstance(x, list)
    return [f(y) for y in x]


def to_class(c: Type[T], x: Any) -> dict:
    assert isinstance(x, c)
    return cast(Any, x).to_dict()


@dataclass
class IM:
    id: Optional[str] = None
    created: Optional[int] = None
    is_archived: Optional[bool] = None
    is_im: Optional[bool] = None
    is_org_shared: Optional[bool] = None
    user: Optional[str] = None
    is_user_deleted: Optional[bool] = None
    priority: Optional[int] = None

    @staticmethod
    def from_dict(obj: Any) -> 'IM':
        assert isinstance(obj, dict)
        id = from_union([from_str, from_none], obj.get("id"))
        created = from_union([from_int, from_none], obj.get("created"))
        is_archived = from_union([from_bool, from_none], obj.get("is_archived"))
        is_im = from_union([from_bool, from_none], obj.get("is_im"))
        is_org_shared = from_union([from_bool, from_none], obj.get("is_org_shared"))
        user = from_union([from_str, from_none], obj.get("user"))
        is_user_deleted = from_union([from_bool, from_none], obj.get("is_user_deleted"))
        priority = from_union([from_int, from_none], obj.get("priority"))
        return IM(id, created, is_archived, is_im, is_org_shared, user, is_user_deleted, priority)

    def to_dict(self) -> dict:
        result: dict = {}
        result["id"] = from_union([from_str, from_none], self.id)
        result["created"] = from_union([from_int, from_none], self.created)
        result["is_archived"] = from_union([from_bool, from_none], self.is_archived)
        result["is_im"] = from_union([from_bool, from_none], self.is_im)
        result["is_org_shared"] = from_union([from_bool, from_none], self.is_org_shared)
        result["user"] = from_union([from_str, from_none], self.user)
        result["is_user_deleted"] = from_union([from_bool, from_none], self.is_user_deleted)
        result["priority"] = from_union([from_int, from_none], self.priority)
        return result


@dataclass
class ResponseMetadata:
    next_cursor: Optional[str] = None
    messages: Optional[List[str]] = None
    warnings: Optional[List[str]] = None

    @staticmethod
    def from_dict(obj: Any) -> 'ResponseMetadata':
        assert isinstance(obj, dict)
        next_cursor = from_union([from_str, from_none], obj.get("next_cursor"))
        messages = from_union([lambda x: from_list(from_str, x), from_none], obj.get("messages"))
        warnings = from_union([lambda x: from_list(from_str, x), from_none], obj.get("warnings"))
        return ResponseMetadata(next_cursor, messages, warnings)

    def to_dict(self) -> dict:
        result: dict = {}
        result["next_cursor"] = from_union([from_str, from_none], self.next_cursor)
        result["messages"] = from_union([lambda x: from_list(from_str, x), from_none], self.messages)
        result["warnings"] = from_union([lambda x: from_list(from_str, x), from_none], self.warnings)
        return result


@dataclass
class IMListResponse:
    ok: Optional[bool] = None
    ims: Optional[List[IM]] = None
    warning: Optional[str] = None
    response_metadata: Optional[ResponseMetadata] = None
    error: Optional[str] = None
    needed: Optional[str] = None
    provided: Optional[str] = None

    @staticmethod
    def from_dict(obj: Any) -> 'IMListResponse':
        assert isinstance(obj, dict)
        ok = from_union([from_bool, from_none], obj.get("ok"))
        ims = from_union([lambda x: from_list(IM.from_dict, x), from_none], obj.get("ims"))
        warning = from_union([from_str, from_none], obj.get("warning"))
        response_metadata = from_union([ResponseMetadata.from_dict, from_none], obj.get("response_metadata"))
        error = from_union([from_str, from_none], obj.get("error"))
        needed = from_union([from_str, from_none], obj.get("needed"))
        provided = from_union([from_str, from_none], obj.get("provided"))
        return IMListResponse(ok, ims, warning, response_metadata, error, needed, provided)

    def to_dict(self) -> dict:
        result: dict = {}
        result["ok"] = from_union([from_bool, from_none], self.ok)
        result["ims"] = from_union([lambda x: from_list(lambda x: to_class(IM, x), x), from_none], self.ims)
        result["warning"] = from_union([from_str, from_none], self.warning)
        result["response_metadata"] = from_union([lambda x: to_class(ResponseMetadata, x), from_none], self.response_metadata)
        result["error"] = from_union([from_str, from_none], self.error)
        result["needed"] = from_union([from_str, from_none], self.needed)
        result["provided"] = from_union([from_str, from_none], self.provided)
        return result


def im_list_response_from_dict(s: Any) -> IMListResponse:
    return IMListResponse.from_dict(s)


def im_list_response_to_dict(x: IMListResponse) -> Any:
    return to_class(IMListResponse, x)
